package ext

import (
	"bytes"
	"context"
	"io/ioutil"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.flipt.io/flipt/rpc/flipt"
	"go.flipt.io/flipt/storage"
)

type mockLister struct {
	flags   []*flipt.Flag
	flagErr error

	segments   []*flipt.Segment
	segmentErr error

	rules   []*flipt.Rule
	ruleErr error
}

func (m mockLister) ListFlags(ctx context.Context, opts ...storage.QueryOption) ([]*flipt.Flag, error) {
	return m.flags, m.flagErr
}

func (m mockLister) ListSegments(ctx context.Context, opts ...storage.QueryOption) ([]*flipt.Segment, error) {
	return m.segments, m.segmentErr
}

func (m mockLister) ListRules(ctx context.Context, flagKey string, opts ...storage.QueryOption) ([]*flipt.Rule, error) {
	return m.rules, m.ruleErr
}

func TestExport(t *testing.T) {
	lister := mockLister{
		flags: []*flipt.Flag{
			{
				Key:         "flag1",
				Name:        "flag1",
				Description: "description",
				Enabled:     true,
				Variants: []*flipt.Variant{
					{
						Id:   "1",
						Key:  "variant1",
						Name: "variant1",
						Attachment: `{
							"pi": 3.141,
							"happy": true,
							"name": "Niels",
							"nothing": null,
							"answer": {
							  "everything": 42
							},
							"list": [1, 0, 2],
							"object": {
							  "currency": "USD",
							  "value": 42.99
							}
						  }`,
					},
					{
						Id:  "2",
						Key: "foo",
					},
				},
			},
		},
		segments: []*flipt.Segment{
			{
				Key:         "segment1",
				Name:        "segment1",
				Description: "description",
				MatchType:   flipt.MatchType_ANY_MATCH_TYPE,
				Constraints: []*flipt.Constraint{
					{
						Id:       "1",
						Type:     flipt.ComparisonType_STRING_COMPARISON_TYPE,
						Property: "foo",
						Operator: "eq",
						Value:    "baz",
					},
					{
						Id:       "2",
						Type:     flipt.ComparisonType_STRING_COMPARISON_TYPE,
						Property: "fizz",
						Operator: "neq",
						Value:    "buzz",
					},
				},
			},
		},
		rules: []*flipt.Rule{
			{
				Id:         "1",
				SegmentKey: "segment1",
				Rank:       1,
				Distributions: []*flipt.Distribution{
					{
						Id:        "1",
						VariantId: "1",
						RuleId:    "1",
						Rollout:   100,
					},
				},
			},
		},
	}

	var (
		exporter = NewExporter(lister)
		b        = new(bytes.Buffer)
	)

	err := exporter.Export(context.Background(), b)
	assert.NoError(t, err)

	in, err := ioutil.ReadFile("testdata/export.yml")
	assert.NoError(t, err)

	assert.YAMLEq(t, string(in), b.String())
}
